@@ -5,10 +5,17 @@
 #define O_NDELAY        FNDELAY /* Non-blocking I/O (4.2 style) */
 #endif
 
+#ifdef __GNUC__
+#define LD_SWITCH_SYSTEM -e __start -static
+#else
 #define LD_SWITCH_SYSTEM -e __start -Bstatic
+#endif
 
 /* In SunOS 4.1, a static function called by tzsetwall reportedly
    clears the byte just past an eight byte region it mallocs, corrupting
    GNU malloc's memory pool.  But Sun's malloc doesn't seem to mind. */
 
 #define SYSTEM_MALLOC
+
+/* Sun4s running SunOS 4+ usually have sound support. */
+#define USE_SOUND
